package com.example.student.service;

import com.example.student.dto.ErrorDto;
import com.example.student.dto.StudentDto;
import com.example.student.dto.TeacherDto;
import com.example.student.feign.FeignService;
import com.example.student.mapper.StudentMapper;
import com.example.student.model.Student;
import com.example.student.repository.StudentRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class StudentService {

    @Autowired
    private StudentRepository repository;

    @Autowired
    private StudentMapper mapper;

    @Autowired
    private FeignService feignService;

    public boolean createAndUpdateStudent(int id, StudentDto studentDto) {
        Optional<Student> student = repository.findById(id).filter(Student::isActive);
        if (student.isPresent() && studentDto != null) {
            Student updatedStudent = mapper.DtoToEntity(studentDto);
            updatedStudent.setId(id);
            updatedStudent.setActive(true);
            repository.save(updatedStudent);
            return true;
        }
        else if (!student.isPresent() && studentDto != null) {
            Student updatedStudent = mapper.DtoToEntity(studentDto);
            updatedStudent.setActive(true);
            repository.save(updatedStudent);
            return true;
        }
        return false;
    }

    public List<StudentDto> findAllStudents() {
        return repository.findAll()
                .stream()
                .filter(Student::isActive)
                .map(x -> mapper.entityToDto(x))
                .collect(Collectors.toList());
    }

    public Optional<StudentDto> findStudentById(int id) {
        Student student = repository.findById(id).filter(Student::isActive).get();
        StudentDto studentDto = mapper.entityToDto(student);
        return Optional.of(studentDto);
    }

    public void deleteStudent(int id) {
        repository.deleteById(id);
    }

    public List<StudentDto> findStudentByAge(int age) {
        return repository.findStudentByAge(age)
                .stream()
                .filter(Student::isActive)
                .map(x -> mapper.entityToDto(x))
                .collect(Collectors.toList());
    }

    public List<StudentDto> findStudentByHobby(String hobby) {
        return repository.findStudentByHobby(hobby)
                .stream()
                .filter(Student::isActive)
                .map(x -> mapper.entityToDto(x))
                .collect(Collectors.toList());
    }

    public List<StudentDto> findStudentByTeacherId(int teacherId) {
        return repository.findByTeacherId(teacherId)
                .stream()
                .filter(Student::isActive)
                .map(x -> mapper.entityToDto(x))
                .collect(Collectors.toList());
    }

    public List<StudentDto> findStudentAfterADateOfBirth(Date dateOfBirth) {
        return repository.findAll()
                .stream()
                .filter(Student::isActive)
                .filter(x -> x.getDateOfBirth().after(dateOfBirth))
                .map(x -> mapper.entityToDto(x))
                .collect(Collectors.toList());
    }

    public List<StudentDto> findStudentBeforeADateOfBirth(Date dateOfBirth) {
        return repository.findAll()
                .stream()
                .filter(Student::isActive)
                .filter(x -> x.getDateOfBirth().before(dateOfBirth))
                .map(x -> mapper.entityToDto(x))
                .collect(Collectors.toList());
    }

    public List<StudentDto> findTeacherByName(String name) {
        TeacherDto teacherDto = feignService.findTeacherByName(name).getBody();
        Integer teacherId = teacherDto.getId();
        if (teacherId == 404) {
//            return new ResponseEntity<>("Teacher service is not available", HttpStatus.OK);
            StudentDto student = new StudentDto(0, null, 0, 0, null,null,0,teacherDto.getErrors());
            List<StudentDto> studentDtoList = new ArrayList<>();
            studentDtoList.add(student);
            return studentDtoList;
        }
        else if (teacherDto.getErrors().size() != 0) {
            StudentDto student = new StudentDto(0, null, 0, 0, null,null,0,teacherDto.getErrors());
            List<StudentDto> studentDtoList = new ArrayList<>();
            studentDtoList.add(student);
            return studentDtoList;
        }
        List<StudentDto> studentList = this.findStudentByTeacherId(teacherId);
        return studentList;
    }

    public List<TeacherDto> findTeacherByAge(int age) {
        return feignService.findTeacherByAge(age).getBody();
    }
}
