package com.example.teacher.controller;


import com.example.teacher.dto.TeacherDto;
import com.example.teacher.exception.ResourceNotFoundCustomException;
import com.example.teacher.model.Teacher;
import com.example.teacher.service.TeacherService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;

@RestController
@RequestMapping("/teachers")
public class TeacherController {

    @Autowired
    private TeacherService teacherService;

    // POST methods
    @PostMapping
    public ResponseEntity<Teacher> createStudent(@RequestBody Teacher teacher) {
        Teacher myStudent = teacherService.createAndUpdateTeacher(teacher);
        return new ResponseEntity<Teacher>(myStudent, HttpStatus.CREATED);
    }

    @GetMapping("/{id}")
    public ResponseEntity<Teacher> findTeacherById(@PathVariable int id) throws ResourceNotFoundCustomException, InterruptedException {
        Optional<Teacher> myTeacher = teacherService.findTeacherById(id);
        if (myTeacher.isPresent()) {
            Teacher teacherBody = myTeacher.get();
            return new ResponseEntity<>(myTeacher.get(), HttpStatus.FOUND);
        }
        return new ResponseEntity<>(null, HttpStatus.NOT_FOUND);
    }

    // GET methods
    @GetMapping("/id-array")
    public ResponseEntity<List<Teacher>> findTeacherByIdArray(@RequestParam int[] id) throws InterruptedException, ResourceNotFoundCustomException {
        List<Teacher> teacherList = new ArrayList<>();
        for (int x: id) {
            Optional<Teacher> myTeacher = teacherService.findTeacherById(x);
            myTeacher.ifPresent(teacherList::add);
        }
        if (teacherList.size() > 0) {
            return new ResponseEntity<List<Teacher>>(teacherList, HttpStatus.FOUND);
        }
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }


//    // Test 1
//    @GetMapping("/id-array-async")
//    public ResponseEntity<List<Teacher>> findTeacherByIdAsync(@RequestParam int[] id) throws ExecutionException, InterruptedException {
//        List<Teacher> teacherList = new ArrayList<>();
//        List<CompletableFuture<Optional<Teacher>>> completableFutureList = new ArrayList<>();
//        for (int x: id) {
//            CompletableFuture<Optional<Teacher>> myCompletableTeacher = teacherService.findTeacherByIdAsync(x);
//            completableFutureList.add(myCompletableTeacher);
//        }
////        completableFutureList.forEach(CompletableFuture::join);
//        for (CompletableFuture<Optional<Teacher>> x: completableFutureList
//             ) {
//            Optional<Teacher> myOptionalTeacher = x.get();
//            myOptionalTeacher.ifPresent(teacherList::add);
//        }
//        if (teacherList.size() > 0) {
//            return new ResponseEntity<List<Teacher>>(teacherList, HttpStatus.FOUND);
//        }
//        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
//    }

    // Test 2
    @GetMapping("/id-array-async")
    public ResponseEntity<List<Teacher>> findTeacherByIdAsync(@RequestParam int[] id) throws ExecutionException, InterruptedException {
        List<Teacher> teacherList = new ArrayList<>();
        List<CompletableFuture<Optional<Teacher>>> completableFutureList = new ArrayList<>();
        for (int x: id) {
            CompletableFuture<Optional<Teacher>> myCompletableTeacher = teacherService.findTeacherByIdAsync(x);
            completableFutureList.add(myCompletableTeacher);
        }
        for (CompletableFuture<Optional<Teacher>> x: completableFutureList
             ) {
            Optional<Teacher> teacherOptional = x.join();
            teacherOptional.ifPresent(teacherList::add);
        }
        if (teacherList.size() > 0) {
            return new ResponseEntity<List<Teacher>>(teacherList, HttpStatus.FOUND);
        }
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }

    @GetMapping("/all")
    public ResponseEntity<List<Teacher>> findAllStudents() {
        List<Teacher> studentList = teacherService.findAllTeachers();
        return new ResponseEntity<List<Teacher>>(studentList, HttpStatus.OK);
    }

    @GetMapping("/name/{name}")
    public ResponseEntity<TeacherDto> findTeacherByName(@PathVariable String name) {
        TeacherDto myTeacher = teacherService.findTeacherByName(name);
        return new ResponseEntity<TeacherDto>(myTeacher, HttpStatus.OK);
    }


    @GetMapping("/age-subject/{age}/{subject}")
    public ResponseEntity<List<Teacher>> findTeacherByAgeAndSubject(@PathVariable int age, @PathVariable String subject) {
        List<Teacher> teacherList = teacherService.findTeacherByAgeAndSubject(age, subject);
        return new ResponseEntity<List<Teacher>>(teacherList, HttpStatus.FOUND);
    }

    @GetMapping("/age/{age}")
    public ResponseEntity<List<Teacher>> findTeacherByAge(@PathVariable int age) {
        List<Teacher> teacherList = teacherService.findTeacherByAge(age);
        return new ResponseEntity<List<Teacher>>(teacherList, HttpStatus.OK);
    }

    // PUT methods
    @PutMapping("/{id}")
    public ResponseEntity<Teacher> updateStudent(@PathVariable int id, @RequestBody Teacher teacher) {
        teacher.setId(id);
        Teacher myTeacher = teacherService.createAndUpdateTeacher(teacher);
        return new ResponseEntity<Teacher>(myTeacher, HttpStatus.ACCEPTED);
    }

    // DELETE methods
    @DeleteMapping("/delete/{id}")
    public ResponseEntity<String> deleteTeacherById(@PathVariable int id) {
        boolean isDeleted = teacherService.deleteTeacherById(id);
        if (isDeleted) {
            return new ResponseEntity<String>("Teacher deleted", HttpStatus.OK);
        }
        return new ResponseEntity<String>("No Teacher found", HttpStatus.NOT_FOUND);

    }

}
