package com.example.teacher.service;

import com.example.teacher.dto.ErrorDto;
import com.example.teacher.dto.TeacherDto;
import com.example.teacher.exception.ResourceNotFoundCustomException;
import com.example.teacher.mapper.TeacherMapper;
import com.example.teacher.model.Teacher;
import com.example.teacher.repository.TeacherRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TeacherService {

    @Autowired
    private TeacherRepository teacherRepository;

    @Autowired
    private TeacherMapper mapper;

    // Create A ThreadPool
    ExecutorService service = Executors.newFixedThreadPool(10);
    public Teacher createAndUpdateTeacher(Teacher teacher) {
        teacher.setActive(true);
        return teacherRepository.save(teacher);
    }

    public List<Teacher> findAllTeachers() {

        return teacherRepository.findAll()
                .stream()
                .filter(Teacher::isActive)
                .collect(Collectors.toList());
    }

    // GET Teacher By ID Synchronously
    public Optional<Teacher> findTeacherById(int id) throws InterruptedException, ResourceNotFoundCustomException {
        Thread.sleep(500);
        log.info("GET Teacher of id " + id + " by Thread " + Thread.currentThread().getName());
        Optional<Teacher> myTeacher = teacherRepository.findById(id);
//        return Optional.of(myTeacher.filter(Teacher::isActive).orElseThrow(() -> new ResourceNotFoundCustomException("Teacher does not exist")));
        if (myTeacher.isPresent() && myTeacher.get().isActive()){
            return myTeacher;
        }
        else {
            throw new ResourceNotFoundCustomException("Teacher does not exist");
        }
//        return Optional.empty();
    }

    // GET Teacher By ID Asynchronously
    @Async
    public CompletableFuture<Optional<Teacher>> findTeacherByIdAsync(int id) throws InterruptedException {
        Thread.sleep(500);
        log.info("GET Teacher of id " + id + " by Thread " + Thread.currentThread().getName());
        Optional<Teacher> myTeacher = teacherRepository.findById(id);
        if (myTeacher.isPresent() && myTeacher.get().isActive()){
            return CompletableFuture.completedFuture(myTeacher);
        }
        return CompletableFuture.completedFuture(Optional.empty());
    }

    public TeacherDto findTeacherByName(String name) {
        Teacher teacher =  teacherRepository.findTeacherByName(name);
        if (teacher == null) {
            ErrorDto errorDto = new ErrorDto("custom error from teacher service", "message from teacher service");
            List<ErrorDto> errorDtoList = new ArrayList<>();
            errorDtoList.add(errorDto);
            TeacherDto teacherDto = new TeacherDto(0, null, 0, null, null, errorDtoList);
            return teacherDto;
        }
        else {
            TeacherDto teacherDto = mapper.entityToDto(teacher);
            return teacherDto;
        }
    }

    public boolean deleteTeacherById(int id) {
        Teacher teacher = teacherRepository.findTeacherById(id);
        boolean isDeleted = false;
        if ((teacher != null) && (teacher.isActive())) {
            isDeleted = true;
            teacher.setActive(false);
            teacherRepository.save(teacher);
        }
        return isDeleted;
    }

    public List<Teacher> findTeacherByAgeAndSubject(int age, String subject) {
        return teacherRepository.findTeacherByAgeAndSubject(age, subject);
    }

    public List<Teacher> findTeacherByAge(int age) {
        return teacherRepository.findTeacherByAge(age);
    }
}
