package com.example.teacher.controller;


import com.example.teacher.model.Teacher;
import io.restassured.RestAssured;
import io.restassured.response.Response;
import io.restassured.specification.RequestSpecification;
import lombok.extern.slf4j.Slf4j;
import org.hamcrest.Matcher;
import org.json.JSONException;
import org.json.JSONObject;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import static org.hamcrest.Matchers.equalTo;
import static org.junit.jupiter.api.Assertions.assertEquals;

@Slf4j
@WebMvcTest
public class TeacherControllerTest {

    @Test
    public void getAllTeachersStatusTest(){
        RestAssured.get("http://localhost:8081/teachers/all").then().statusCode(200).log().all();
    }

    @Test
    public void getTeacherByWrongIdStatusTest() {
        RestAssured.get("http://localhost:8081/teachers/411").then().statusCode(404).log().all();
    }

    @Test
    public void getTeacherByIdTest() throws ParseException {
        Teacher myTeacher = new Teacher(1, "sahal", 25, "English", new Date(), true);

        Response response = RestAssured
                .get("http://localhost:8081/teachers/1")
                .then()
                .statusCode(302)
                .assertThat()
                .body("id", equalTo(myTeacher.getId()))
                .body("name", equalTo(myTeacher.getName()))
                .body("age", equalTo(myTeacher.getAge()))
                .body("dateOfJoining", equalTo("11:01:2003"))
                .body("subject", equalTo(myTeacher.getSubject()))
                .extract()
                .response();
        log.info(response.asPrettyString());
    }

    @Test
    public void getTeacherByWrongIdTest() {
        Response response = RestAssured
                .get("http://localhost:8081/teachers/999")
                .then()
                .statusCode(404)
                .extract().response();

        Assert.assertEquals(response.asString(), "Teacher does not exist");
        log.info(response.asPrettyString());
    }

    @Test
    public void createTeacherTest() throws JSONException {
        Date date = new Date();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd-MM-yyyy");
        String stringDate = simpleDateFormat.format(date);
        RestAssured.baseURI = "http://localhost:8081/teachers";
        RequestSpecification request = RestAssured.given();
        JSONObject requestParams = new JSONObject();
        requestParams.put("name", "Qadeer");
        requestParams.put("age", "57");
        requestParams.put("subject", "Math");
        requestParams.put("dateOfJoining", stringDate);
        request.header("Content-Type", "application/json");
        request.body(requestParams.toString());
        Response response = request.post();
        System.out.println(response.statusCode());
        assertEquals(response.statusCode(), 201);
    }
}
